package com.meterware.httpunit;
/********************************************************************************************************************
* $Id: HtmlTablesTest.java,v 1.7 2000/09/01 14:54:07 russgold Exp $
*
* Copyright (c) 2000, Russell Gold
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
* documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
* the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
* to permit persons to whom the Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial portions 
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
* THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
* CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*
*******************************************************************************************************************/
import java.net.URL;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


/**
 * A unit test of the httpunit parsing classes.
 **/
public class HtmlTablesTest extends TestCase {

	public static void main(String args[]) {
        junit.textui.TestRunner.run( suite() );
	}
	
	
	public static Test suite() {
		return new TestSuite( HtmlTablesTest.class );
	}


	public HtmlTablesTest( String name ) {
	    super( name );
	}


    public void setUp() throws Exception {
        _baseURL = new URL( "http://www.meterware.com" );
        _noTables  = new ReceivedPage( _baseURL, HEADER + "<body>This has no tables but it does" +
                                       "have <a href=\"/other.html\">an active link</A>" +
                                       " and <a name=here>an anchor</a>" +
                                       "</body></html>" );
        _oneTable  = new ReceivedPage( _baseURL, HEADER + "<body><h2>Interesting data</h2>" +
                                       "<table summary=\"tough luck\">" +
                                       "<tr><td>One</td><td>&nbsp;</td><td>1</td></tr>" +
                                       "<tr><td colspan=3><IMG SRC=\"/images/spacer.gif\" ALT=\"\" WIDTH=1 HEIGHT=1></td></tr>" + 
                                       "<tr><td>Two</td><td>&nbsp;</td><td>2</td></tr>" +
                                       "<tr><td colspan=3><IMG SRC=\"/images/spacer.gif\" ALT=\"\" WIDTH=1 HEIGHT=1></td></tr>" + 
                                       "<tr><td>Three</td><td>&nbsp;</td><td>3</td></tr>" +
                                       "</table></body></html>" );
        _nestTable = new ReceivedPage( _baseURL, HEADER + "<body><h2>Interesting data</h2>" +
                                       "<table summary=\"outer one\">" +
                                       "<tr><td>" +
                                       "Inner Table<br>" +
                                       "<table summary=\"inner one\">" +
                                       "        <tr><td>Red</td><td>1</td></tr>" +
                                       "        <tr><td>Blue</td><td>2</td></tr>" +
                                       "</table></td></tr>" +
                                       "</table></body></html>" );
        _spanTable = new ReceivedPage( _baseURL, HEADER + "<body><h2>Interesting data</h2>" +
                                       "<table summary=\"tough luck\">" +
                                       "<tr><th colspan=2>Colors</th><th>Names</th></tr>" +
                                       "<tr><td>Red</td><td rowspan=\"2\"><b>gules</b></td><td>rot</td></tr>" +
                                       "<tr><td>Green</td><td><a href=\"nowhere\">vert</a></td></tr>" +
                                       "</table></body></html>" );
        _badTable  = new ReceivedPage( _baseURL, HEADER + "<body><h2>Interesting data</h2>" +
                                       "<table summary=\"tough luck\">" +
                                       "<tr><th colspan=2>Colors</th><th>Names</th></tr>" +
                                       "<tr><td>Red</td><td rowspan=\"2\"><b>gules</b></td></tr>" +
                                       "<tr><td>Green</td><td><a href=\"nowhere\">vert</a></td></tr>" +
                                       "</table></body></html>" );
    }
	
	
	public void testFindNoTables() {
        WebTable[] tables = _noTables.getTables();
        assertNotNull( tables );
        assertEquals( 0, tables.length );
    }


    public void testFindOneTable() {
        WebTable[] tables = _oneTable.getTables();
        assertEquals( 1, tables.length );
    }


    public void testFindTableSize() {
        WebTable table = _oneTable.getTables()[0];
        assertEquals( 5, table.getRowCount() );
        assertEquals( 3, table.getColumnCount() );
        try {
            table.getCellAsText( 5, 0 );
            fail( "Should throw out of range exception" );
        } catch (IndexOutOfBoundsException e ) {
        }
        try {
            table.getCellAsText( 0, 3 );
            fail( "Should throw out of range exception" );
        } catch (RuntimeException e ) {
        }
    }


    public void testFindTableCell() {
        WebTable table = _oneTable.getTables()[0];
        assertEquals( "Two", table.getCellAsText( 2, 0 ) );
        assertEquals( "3",   table.getCellAsText( 4, 2 ) );
    }


    public void testTableAsText() {
       WebTable table = _oneTable.getTables()[0];
       table.purgeEmptyCells();
       String[][] text = table.asText();
       assertEquals( "rows with text", 3, text.length );
       assertEquals( "Two", text[1][0] );
       assertEquals( "3", text[2][1] );
       assertEquals( "columns with text", 2, text[0].length );
    }



    public void testNestedTable() {
        WebTable[] tables = _nestTable.getTables();
        assertEquals( "top level tables count", 1, tables.length );
        assertEquals( "rows", 1, tables[0].getRowCount() );
        assertEquals( "columns", 1, tables[0].getColumnCount() );
        WebTable[] nested = tables[0].getTableCell( 0, 0 ).getTables();
        assertEquals( "nested tables count", 1, nested.length );
        assertEquals( "nested rows", 2, nested[0].getRowCount() );
        assertEquals( "nested columns", 2, nested[0].getColumnCount() );

        String nestedString = tables[0].getCellAsText( 0, 0 );
        assert( "Cannot find 'Red' in string", nestedString.indexOf( "Red" ) >= 0 );
        assert( "Cannot find 'Blue' in string", nestedString.indexOf( "Blue" ) >= 0 );
    }


    public void testColumnSpan() {
        WebTable table = _spanTable.getTables()[0];
        assertEquals( "Colors", table.getCellAsText( 0, 0 ) );
        assertEquals( "Colors", table.getCellAsText( 0, 1 ) );
        assertEquals( "Names",  table.getCellAsText( 0, 2 ) );
    }

    public void testRowSpan() {
        WebTable table = _spanTable.getTables()[0];
        assertEquals( 3, table.getRowCount() );
        assertEquals( 3, table.getColumnCount() );
        assertEquals( "gules", table.getCellAsText( 1, 1 ) );
        assertEquals( "gules", table.getCellAsText( 2, 1 ) );
        assertEquals( "vert",  table.getCellAsText( 2, 2 ) );
    }

    public void testMissingColumns() {
        WebTable table = _badTable.getTables()[0];
        table.purgeEmptyCells();
        assertEquals( 3, table.getRowCount() );
        assertEquals( 3, table.getColumnCount() );
    }


    public void testInnerTableSeek() throws Exception {
        ReceivedPage rp = new ReceivedPage( _baseURL, HEADER + "<body><h2>Interesting data</h2>" +
                                            "<table summary=\"outer one\">" +
                                            "<tr><td>&nbsp;</td><td>" +
                                            "Inner Table<br>" +
                                            "<table summary=\"inner one\">" +
                                            "        <tr><td colspan=2>&nbsp;</td></tr>" +
                                            "        <tr><td>Red</td><td>1</td></tr>" +
                                            "        <tr><td>Blue</td><td>2</td></tr>" +
                                            "</table></td></tr>" +
                                            "</table></body></html>" );

        String[][] cells = rp.getTableStartingWith( "Red" ).asText();
        assertEquals( "Non-blank rows",    2, cells.length );
        assertEquals( "Non-blank columns", 2, cells[0].length );
        assertEquals( "cell at 1,0",       "Blue", cells[1][0] );
        cells = rp.getTableStartingWithPrefix( "Re" ).asText();
        assertEquals( "Non-blank rows",    2, cells.length );
        assertEquals( "Non-blank columns", 2, cells[0].length );
        assertEquals( "cell at 1,0",       "Blue", cells[1][0] );
    }


    public void testSpanOverEmptyColumns() throws Exception {
        ReceivedPage rp = new ReceivedPage( _baseURL, HEADER + "<body><h2>Interesting data</h2>" +
                                            "<table summary=little>" +
                                            "<tr><td colspan=2>Title</td><td>Data</td></tr>" +
                                            "<tr><td>Name</td><td>&nbsp;</td><td>Value</td></tr>" +
                                            "<tr><td>Name</td><td>&nbsp;</td><td>Value</td></tr>" +
                                            "</table></body></html>" );

        String[][] cells = rp.getTableStartingWith( "Title" ).asText();
        assertEquals( "Non-blank rows",    3, cells.length );
        assertEquals( "Non-blank columns", 2, cells[0].length );
        assertEquals( "cell at 1,1",       "Value", cells[1][1] );
    }


    public void testSpanOverAllEmptyColumns() throws Exception {
        ReceivedPage rp = new ReceivedPage( _baseURL, HEADER + "<body><h2>Interesting data</h2>" +
                                            "<table summary=little>" +
                                            "<tr><td colspan=2>Title</td><td>Data</td></tr>" +
                                            "<tr><td>&nbsp;</td><td>&nbsp;</td><td>Value</td></tr>" +
                                            "<tr><td>&nbsp;</td><td>&nbsp;</td><td>Value</td></tr>" +
                                            "</table></body></html>" );

        String[][] cells = rp.getTableStartingWith( "Title" ).asText();
        assertEquals( "Non-blank rows",    3, cells.length );
        assertEquals( "Non-blank columns", 2, cells[0].length );
        assertEquals( "cell at 1,1",       "Value", cells[1][1] );
    }


    private final static String HEADER = "<html><head><title>A Sample Page</title></head>";
    private ReceivedPage _noTables;
    private ReceivedPage _oneTable;
    private ReceivedPage _nestTable;
    private ReceivedPage _spanTable;
    private ReceivedPage _badTable;
    private URL          _baseURL;
}

